import os
import pandas as pd

try:
    DEBUG = os.environ['DEBUG']
except KeyError:
    DEBUG = False


DATE_FORMAT = "%-d-%b-%y"


def process_to_json(df):
    """
    Returns a json, consisting of the data in the dataframe

    Args:
        df (pandas DataFrame): dataframe comming directly from the db query
    """

    df['date'] = pd.to_datetime(df['date'], errors='coerce')
    df['date'] = df['date'].dt.strftime(DATE_FORMAT)

    all_dates = df['date'].unique()
    all_regions = df['region'].unique()

    df.set_index(['region', 'date'], inplace=True)
    df.sort_index()

    by_region = {}

    for region in all_regions:
        by_region[region] = {}
        for date in all_dates:
            if (region, date) in df.index:
                by_region[region][date] = {
                    "t": df.at[(region, date), 't'],
                    "b": df.at[(region, date), 'b'],
                    "m": df.at[(region, date), 'm'],
                    "ch": df.at[(region, date), 'ch'],
                }
    return by_region
