import os

try:
    DEBUG = os.environ['DEBUG']
except KeyError:
    DEBUG = False

# Function of pareto optimal
def pareto_optimal(employees_all, nv_values, betau, betav, contact_rate, home_efficiency_rate, infection_probability,
                   test_interval):
    current_prod = home_efficiency_rate * employees_all
    current_prod = 100 * (current_prod / employees_all)  # convert values to percentage

    pr_values = []  # Occupancy list (presence rates)
    productivity = []  # productivity list, as output_values
    d_infections = []  # expected number of infections before detecting the first infection
    pr_value = 0
    pr_values.append(pr_value)
    productivity.append(current_prod)
    d_infections.append(0)

    # Divide a day into scale(=2) time intervals 
    # such that (kappa/scale) contacts per interval is possible

    scale = 2
    # Note: Do analysis for (day/scale) intervals and kappa/scale contacts
    tau_bar = int(scale * (5 / 7) * (test_interval / 2))
    infection_probability = infection_probability / scale

    while pr_value < 100:
        pr_value = pr_value + 1
        occupancy = pr_value / 100  # percentage of employee work at offices, occupancy
        # 5 contacts is a constant plus some percentage of presented staff
        kappa = 2 + (contact_rate * 0.3 * occupancy * employees_all)
        kappa = kappa / scale
        alpha = (kappa * (1 - (1 / (employees_all - 1))))
        pu_values = 0
        pv_values = 0
        n_infect = 0
        for d_value in range(1, tau_bar + 1):
            a_value = pow(1 - betau, occupancy * occupancy * kappa * (1 / (employees_all - 1)))
            denom_b = ((employees_all - nv_values - 1) / (employees_all - 1))
            b_value = pow(1 - pu_values * betau, alpha * occupancy * denom_b)
            c_value = pow(1 - pv_values * betau, alpha * occupancy * (nv_values / (employees_all - 1)))

            # (1-pu_values) is the probability remaining health till yesterday, (d_value-1)
            pr_healthy = (a_value * b_value * c_value) * (1 - pu_values)
            pu_values = 1 - pr_healthy

            a_value = pow(1 - betav, occupancy * occupancy * kappa * (1 / (employees_all - 1)))
            b_value = pow(1 - pu_values * betav, alpha *
                          occupancy * ((employees_all - nv_values - 1) / (employees_all - 1)))
            c_value = pow(1 - pv_values * betav, alpha * occupancy * (nv_values / (employees_all - 1)))

            pr_healthy = (a_value * b_value * c_value) * (1 - pv_values)
            pv_values = 1 - pr_healthy

            m_value = 1 + (employees_all - nv_values - 1) * pu_values + (nv_values * pv_values)
            x_value = m_value * (1 - pow(1 - infection_probability, (occupancy * employees_all)))
            n_infect = n_infect + x_value - ((n_infect * x_value) / employees_all)

        n_infect = n_infect * (7 / (tau_bar / scale))  # weekly scaling

        min(n_infect, employees_all)
        if DEBUG:
            print(pr_value, n_infect)

        prob_down = (1 - occupancy) * (employees_all - n_infect)
        current_prod = occupancy * (employees_all - n_infect)
        current_prod += home_efficiency_rate * prob_down
        current_prod = 100 * (current_prod / employees_all)

        pr_values.append(pr_value)
        productivity.append(current_prod)
        d_infections.append(100 * (n_infect / employees_all))

    return pr_values, productivity, d_infections


####################################  Input parameters
# out_pr_values, out_productivity, out_rate_infection = pareto_optimal(employees_all, nv_values, betau, betav, contact_rate, home_efficiency_rate, infection_probability, test_interval)
# out_pr_values, out_productivity, out_rate_infection = pareto_optimal(200, 140, 0.1, 0.03, 0.10, 0.75, 0.002, 7)
if __name__ == "__main__":
    import sys

    if len(sys.argv) < 9:
        outText = f'Please insert 8 required parameters on {sys.argv[0]}'
        sys.exit(outText)

    p1 = int(sys.argv[1])
    p2 = int(sys.argv[2])
    p3 = float(sys.argv[3])
    p4 = float(sys.argv[4])
    p5 = float(sys.argv[5])
    p6 = float(sys.argv[6])
    p7 = float(sys.argv[7])
    p8 = float(sys.argv[8])

    out_pr_values, out_productivity, out_rate_infection = pareto_optimal(p1, p2, p3, p4, p5, p6, p7, p8)

    print("pr_value, productivity, rate_infection")
    for col1, col2, col3 in zip(out_pr_values, out_productivity, out_rate_infection):
        print(f"{col1}, {col2}, {col3}")
