import os
import numpy as np
import pandas as pd
import geopandas as gpd
import json
import scipy.stats
import datetime


try:
    DEBUG = os.environ['DEBUG']
except KeyError:
    DEBUG = False

# date constants
DATE_START = datetime.date(2021, 1, 3)
DATE_END = datetime.date(2022, 5, 22)

# Prepare data on the server start
dists = json.load(open('regioncorrelation_source/data/dists.json'))

# pre-calculate the incidence change
region_incidence = json.load(
    open('regioncorrelation_source/data/regions-incidence.json'))
diff_dict = {}

# list of all region ids
rids_all = [r['region'] for r in region_incidence]

# region info
region_list = {region['region']: {
    'id': region['region'],
    'population': region['population'],
    'name': region['name'],
    'country': region['country'],
} for region in region_incidence}


for region in region_incidence:
    rcases = np.array(region['incidence'])
    rcases_diff = np.diff(rcases)
    rcases_diff_pos = [a if a > 0 else 0 for a in rcases_diff]
    diff_dict[region['region']] = rcases_diff_pos

# find all neighboring regions
region_neigs_all = {r['region']: list(
    dists[r['region']].keys()) for r in region_incidence}
region_neigs_in = {r['region']: list(
    r2 for r2 in dists[r['region']].keys() if r2[:2] == r['region'][:2]) for r in region_incidence}
region_neigs_out = {r['region']: list(
    r2 for r2 in dists[r['region']].keys() if r2[:2] != r['region'][:2]) for r in region_incidence}


def corr(a1, a2):
    corr_val = scipy.stats.pearsonr(a1, a2)[0]
    return 0 if np.isnan(corr_val) else corr_val


def perc(val):
    return int(100 * val)


def correlation_timeline():
    week_delta = datetime.timedelta(days=7)
    date_this = DATE_START
    date_list = []
    while date_this <= DATE_END:
        date_list.append(date_this.strftime('%Y-%m-%d'))
        date_this += week_delta
    return date_list


def correlation_regionlist():
    return region_list


def correlation_map(time_from, time_to, mode):
    time_diff = 1
    # inc_dict = {rid: diff_dict[rid][time_from:time_to]
    #             for rid in rids_all}
    # vals_export_dict = diff_dict[rid2][time_from:time_to]

    print(len(diff_dict['pl-3261']))

    # average incidence
    if mode == 'inc':
        return {rid: int(np.array(diff_dict[rid][time_from:time_to]).mean()) for rid in rids_all}

    # average change of incidence
    if mode == 'ch':
        return {rid: int(np.array(diff_dict[rid][time_from:time_to]).mean()) for rid in rids_all}

    else:
        # average change of incidence
        inc_import_dict = {rid: diff_dict[rid][time_from + time_diff:time_to]
                           for rid in rids_all}
        inc_export_dict = {rid: diff_dict[rid][time_from:time_to - time_diff]
                           for rid in rids_all}

        # export correlation
        if mode == 'c_e':
            return {rid:
                    perc(
                        np.array(
                            [corr(inc_export_dict[rid], inc_import_dict[rid2])
                             for rid2 in region_neigs_all[rid]]
                        ).mean()
                    )
                    for rid in rids_all}

        # import correlation
        if mode == 'c_i':
            return {rid:
                    perc(
                        np.array(
                            [corr(inc_import_dict[rid], inc_export_dict[rid2])
                             for rid2 in region_neigs_all[rid]]
                        ).mean()
                    )
                    for rid in rids_all}

        # border export effect
        if mode == 'b_i':
            border_ids = [rid for rid in rids_all if rids_all if len(
                region_neigs_out[rid]) > 1]

            corrs_import_in = {rid: perc(
                np.array(
                    [corr(inc_import_dict[rid], inc_export_dict[rid2])
                     for rid2 in region_neigs_in[rid]]
                ).mean()) for rid in border_ids}

            corrs_import_out = {rid: perc(
                np.array(
                    [corr(inc_import_dict[rid], inc_export_dict[rid2])
                     for rid2 in region_neigs_out[rid]]
                ).mean()) for rid in border_ids}

            return {rid: corrs_import_in[rid] - corrs_import_out[rid] for rid in border_ids}

        # border import effect
        if mode == 'b_e':
            border_ids = [rid for rid in rids_all if rids_all if len(
                region_neigs_out[rid]) > 1]

            corrs_export_in = {rid: perc(
                np.array(
                    [corr(inc_export_dict[rid], inc_import_dict[rid2])
                     for rid2 in region_neigs_in[rid]]
                ).mean()) for rid in border_ids}

            corrs_export_out = {rid: perc(
                np.array(
                    [corr(inc_export_dict[rid], inc_import_dict[rid2])
                     for rid2 in region_neigs_out[rid]]
                ).mean()) for rid in border_ids}
            return {rid: corrs_export_in[rid] - corrs_export_out[rid] for rid in border_ids}

    # corrs_import = {rid: {
    #     rid2: corr(
    #         inc_import_dict[rid], inc_export_dict[rid2]) for rid2 in region_neigs_all[rid]
    # } for rid in rids_all}

    # corrs_export = {rid: {
    #     rid2: corr(
    #         inc_export_dict[rid], inc_import_dict[rid2]) for rid2 in region_neigs_all[rid]
    # } for rid in rids_all}

    # corrs_syn = {rid: {
    #     rid2: corr(
    #         inc_dict[rid], inc_dict[rid2]) for rid2 in region_neigs_all[rid]
    # } for rid in rids_all}

    # print(inc_dict['de-08226'])

    # for region in region_incidence:
    #     rid = region['region']
    #     r_diffs = diff_dict[rid][time_from: time_to]
    #     r_incs = region['incidence'][time_from: time_to]

    #     # calculate all correlations
    #     r_neigh_ids_all = region_neigs_all[rid]
    #     r_neigh_ids_in = region_neigs_in[rid]
    #     r_neigh_ids_out = region_neigs_out[rid]

        # diff_r_import = diff_dict[rid][time_from + time_diff:time_to]
        # diff_r_export = diff_dict[rid][time_from:time_to - time_diff]

        # corrs_import_all = {
        #     rid2: corrs_import[rid][rid2] for rid2 in r_neigh_ids_all}

        # corrs_export_all = {
        #     rid2: corrs_export[rid][rid2] for rid2 in r_neigh_ids_all}

        # corrs_syn_all = {rid2:
        #                  corrs_syn[rid][rid2] for rid2 in r_neigh_ids_all}

        # if len(r_neigh_ids_out) > 1:
        #     corrs_import_in = [corrs_import[rid][rid2]
        #                        for rid2 in r_neigh_ids_in]

        #     # corrs_export_in = [corrs_export[rid][rid2]
        #     #                    for rid2 in r_neigh_ids_in]

        #     corrs_import_out = [corrs_import[rid][rid2]
        #                         for rid2 in r_neigh_ids_out]

        #     # corrs_export_out = [corrs_export[rid][rid2]
        #     #                     for rid2 in r_neigh_ids_out]

        #     data[rid]['b_i'] = perc(np.array(
        #         corrs_import_in).mean() - np.array(corrs_import_out).mean())
        #     data[rid]['b_e'] = perc(np.array(
        #         corrs_export_in).mean() - np.array(corrs_export_out).mean())

        # data[rid]['c_i'] = perc(
        #     np.array(list(corrs_import_all.values())).mean())
        # data[rid]['c_e'] = perc(
        #     np.array(list(corrs_export_all.values())).mean())
        # data[rid]['c_s'] = perc(
        #     np.array(list(corrs_syn_all.values())).mean())

        # border effect
        # autocorr. factor
        # hotspots ?

    return {}


def correlations_region(region_id, time_diff, time_from, time_to):
    """
    Returns correlations of neighboring regions from region_id in time from time_from to time_end
    Args:
        region_id(string):
        time_diff(number);
        time_from(number):
        time_to(number):
    """

    r_dists = dists[region_id]
    r_neigh_ids = region_neigs_all[region_id]
    diff_r_import = diff_dict[region_id][time_from + time_diff:time_to]
    diff_r_export = diff_dict[region_id][time_from:time_to - time_diff]
    diff_r_sim = diff_dict[region_id][time_from:time_to]

    # cases_dict[r_neigh_ids[0]]
    out = {}
    for rn_id in r_neigh_ids:
        diff_import = diff_dict[rn_id][time_from:time_to - time_diff]
        diff_export = diff_dict[rn_id][time_from + time_diff:time_to]
        diff_syn = diff_dict[rn_id][time_from:time_to]

        corr_import = scipy.stats.pearsonr(diff_r_import, diff_import)[0]
        corr_export = scipy.stats.pearsonr(diff_r_export, diff_export)[0]
        corr_syn = scipy.stats.pearsonr(diff_r_sim, diff_syn)[0]

        dist = r_dists[rn_id]

        out[rn_id] = {
            'c_i': corr_import,
            'c_e': corr_export,
            'c_s': corr_syn,
            'dist': dist,
            'rid': rn_id,
            # 'cases': cases_dict[rn_id],
        }
    return out


if __name__ == "__main__":
    import sys
