import os
import math
import copy
from random import randint
from cmath import exp

try:
    DEBUG = os.environ['DEBUG']
except KeyError:
    DEBUG = False


def Probability_of_Infection_one_resident_after_d_days(m, kappa, beta, d):
    if m == 1:
        return 1
    q = m/(1+(m-1)*pow(1-beta, kappa*d))  # SIR
    q = q-1
    q = q/(m-1)
    return q



def Probability_of_Infection_group_size_r_after_d_days(m, kappa, beta, d, r):
    x = Probability_of_Infection_one_resident_after_d_days(m, kappa, beta, d)
    prob = (r/m)+(1-(r/m))*(1-pow(1-x, r))
    return prob


def Compute_Exp_detect_time_for_t0(k, tau, Size, Days, m, kappa, beta, t0):
    m2 = m

    sum = 0
    q = 1
    for i in range(k):
        prob = Probability_of_Infection_group_size_r_after_d_days(
            m2, kappa, beta, Days[i]-t0, Size[i])
        # This 0.5 is simulating Continuous time by discretized version
        sum = sum+q*prob * (Days[i]-t0-0.5)
        # We may interpret if a residint get infected at day  t0
        # and do a test 10 days later, the detection time is 9.5 not 10
        q = q*(1-prob)
        m2 = m2-Size[i]

    return sum  # exp_time


def Compute_Exp_detect_time(k, tau, G, D, m, kappa, beta):
    sum = 0
    for i in range(k):
        D_Conf = list()
        S_Conf = list()
        if i == 0:
            D_Conf = copy.copy(D)
            S_Conf = copy.copy(G)
        else:
            for j in range(i, k):
                D_Conf.append(D[j]-D[i-1])
                S_Conf.append(G[j])
            for j in range(0, i):
                D_Conf.append(D[j]+tau-D[i-1])
                S_Conf.append(G[j])
        for t0 in range(D_Conf[0]):
            sum = sum + \
                Compute_Exp_detect_time_for_t0(
                    k, tau, S_Conf, D_Conf, m, kappa, beta, t0)
    sum = sum/tau
    return sum  # exp_time


def Optimal_Configuration(k, tau, m):
    G = list()                 # size of each group
    D = list()                  # testing day of each group

    sum = 0
    for i in range(1, k+1):
        d = (i/k)*tau
        d = math.ceil(d)
        D.append(d)

        s = (1/k)*m
        s = math.floor(s)
        sum = sum+s
        if (sum < (i/k)*m and sum+1 <= (i/k)*m):
            s = s+1
            sum = sum+1

        G.append(s)

    G = Refine(G, m)

    return G, D


def Optimal_Configuration_SearchAll(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group

    if k == 1:
        G_opt.append(m)
        D_opt.append(1)
    if k == 2:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k2(
            k, tau, m, kappa, beta, Max_g)
    if k == 3:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k3(
            k, tau, m, kappa, beta, Max_g)
    if k == 4:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k4(
            k, tau, m, kappa, beta, Max_g)
    if k == 5:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k5(
            k, tau, m, kappa, beta, Max_g)
    if k == 6:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k6(
            k, tau, m, kappa, beta, Max_g)
    if k == 7:
        G_opt, D_opt = Optimal_Configuration_SearchAll_k7(
            k, tau, m, kappa, beta, Max_g)

    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k2(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, Max_g+1):
        s2 = m-s1
        if(s2 <= Max_g):
            for d1 in range(1, tau):
                d2 = tau
                G = list()
                D = list()
                G.append(s1)
                G.append(s2)
                D.append(d1)
                D.append(d2)
                exp_time = Compute_Exp_detect_time(
                    k, tau, G, D, m, kappa, beta)
                if exp_time < min_exp_time:
                    min_exp_time = exp_time
                    G_opt = G
                    D_opt = D

    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k3(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, m-1):
        for s2 in range(1, m):
            if(s1 <= Max_g and s2 <= Max_g and s1+s2 < m):
                s3 = m-(s1+s2)
                if(s3 <= Max_g):
                    for d1 in range(1, tau-1):
                        for d2 in range(d1+1, tau):
                            d3 = tau
                            G = list()
                            D = list()
                            G.append(s1)
                            G.append(s2)
                            G.append(s3)
                            D.append(d1)
                            D.append(d2)
                            D.append(d3)
                            exp_time = Compute_Exp_detect_time(
                                k, tau, G, D, m, kappa, beta)
                            if exp_time < min_exp_time:
                                min_exp_time = exp_time
                                G_opt = G
                                D_opt = D
    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k4(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, m-2):
        for s2 in range(1, m-1):
            for s3 in range(1, m):
                if(s1 <= Max_g and s2 <= Max_g and s3 <= Max_g and s1+s2+s3 < m):
                    s4 = m-(s1+s2+s3)
                    if(s4 <= Max_g):
                        for d1 in range(1, tau-2):
                            for d2 in range(d1+1, tau-1):
                                for d3 in range(d2+1, tau):
                                    d4 = tau
                                    G = list()
                                    D = list()
                                    G.append(s1)
                                    G.append(s2)
                                    G.append(s3)
                                    G.append(s4)
                                    D.append(d1)
                                    D.append(d2)
                                    D.append(d3)
                                    D.append(d4)
                                    exp_time = Compute_Exp_detect_time(
                                        k, tau, G, D, m, kappa, beta)
                                    if exp_time < min_exp_time:
                                        min_exp_time = exp_time
                                        G_opt = G
                                        D_opt = D

    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k5(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, m-3):
        for s2 in range(1, m-2):
            for s3 in range(1, m-1):
                for s4 in range(1, m):
                    if(s1 <= Max_g and s2 <= Max_g and s3 <= Max_g and s4 <= Max_g and s1+s2+s3+s4 < m):
                        s5 = m-(s1+s2+s3+s4)
                        if(s5 <= Max_g):
                            for d1 in range(1, tau-3):
                                for d2 in range(d1+1, tau-2):
                                    for d3 in range(d2+1, tau-1):
                                        for d4 in range(d3+1, tau):
                                            d5 = tau
                                            G = list()
                                            D = list()
                                            G.append(s1)
                                            G.append(s2)
                                            G.append(s3)
                                            G.append(s4)
                                            G.append(s5)
                                            D.append(d1)
                                            D.append(d2)
                                            D.append(d3)
                                            D.append(d4)
                                            D.append(d5)
                                            exp_time = Compute_Exp_detect_time(
                                                k, tau, G, D, m, kappa, beta)
                                            if exp_time < min_exp_time:
                                                min_exp_time = exp_time
                                                G_opt = G
                                                D_opt = D

    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k6(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, m-4):
        for s2 in range(1, m-3):
            for s3 in range(1, m-2):
                for s4 in range(1, m-1):
                    for s5 in range(1, m):
                        if(s1 <= Max_g and s2 <= Max_g and s3 <= Max_g and s4 <= Max_g and s5 <= Max_g and s1+s2+s3+s4+s5 < m):
                            s6 = m-(s1+s2+s3+s4+s5)
                            if(s6 <= Max_g):
                                for d1 in range(1, tau-4):
                                    for d2 in range(d1+1, tau-3):
                                        for d3 in range(d2+1, tau-2):
                                            for d4 in range(d3+1, tau-1):
                                                for d5 in range(d4+1, tau):
                                                    d6 = tau
                                                    G = list()
                                                    D = list()
                                                    G.append(s1)
                                                    G.append(s2)
                                                    G.append(s3)
                                                    G.append(s4)
                                                    G.append(s5)
                                                    G.append(s6)

                                                    D.append(d1)
                                                    D.append(d2)
                                                    D.append(d3)
                                                    D.append(d4)
                                                    D.append(d5)
                                                    D.append(d6)
                                                    exp_time = Compute_Exp_detect_time(
                                                        k, tau, G, D, m, kappa, beta)
                                                    if exp_time < min_exp_time:
                                                        min_exp_time = exp_time
                                                        G_opt = G
                                                        D_opt = D

    return G_opt, D_opt


def Optimal_Configuration_SearchAll_k7(k, tau, m, kappa, beta, Max_g):
    G_opt = list()                    # size of each group
    D_opt = list()                    # testing day of each group
    min_exp_time = tau+1

    for s1 in range(1, m-5):
        for s2 in range(1, m-4):
            for s3 in range(1, m-3):
                for s4 in range(1, m-2):
                    for s5 in range(1, m-1):
                        for s6 in range(1, m):
                            if(s1 <= Max_g and s2 <= Max_g and s3 <= Max_g and s4 <= Max_g and s5 <= Max_g and s6 <= Max_g and s1+s2+s3+s4+s5+s6 < m):
                                s7 = m-(s1+s2+s3+s4+s5+s6)
                                if(s7 <= Max_g):
                                    for d1 in range(1, tau-5):
                                        for d2 in range(d1+1, tau-4):
                                            for d3 in range(d2+1, tau-3):
                                                for d4 in range(d3+1, tau-2):
                                                    for d5 in range(d4+1, tau-1):
                                                        for d6 in range(d5+1, tau):
                                                            d7 = tau
                                                            G = list()
                                                            D = list()
                                                            G.append(s1)
                                                            G.append(s2)
                                                            G.append(s3)
                                                            G.append(s4)
                                                            G.append(s5)
                                                            G.append(s6)
                                                            G.append(s7)

                                                            D.append(d1)
                                                            D.append(d2)
                                                            D.append(d3)
                                                            D.append(d4)
                                                            D.append(d5)
                                                            D.append(d6)
                                                            D.append(d7)
                                                            exp_time = Compute_Exp_detect_time(
                                                                k, tau, G, D, m, kappa, beta)
                                                            if exp_time < min_exp_time:
                                                                min_exp_time = exp_time
                                                                G_opt = G
                                                                D_opt = D

    return G_opt, D_opt


def Refine(H, m):
    G = copy.deepcopy(H)
    Flag = 0
    while Flag == 0:
        sum = 0
        for i in range(len(G)):
            sum = sum+G[i]
        if sum < m:
            min = G[0]
            min_index = 0
            for i in range(len(G)):
                if G[i] < min:
                    min = G[i]
                    min_index = i
            G[min_index] = G[min_index]+1
        else:
            Flag = 1
    return G


def Next_List(Changable, A):
    finish = 1
    for i in range(len(A)):
        if (Changable[i] == 1 and A[i] == 1):
            A[i] = 0
        elif (Changable[i] == 1 and A[i] == 0):
            break
    while (i < len(A) and Changable[i] == 0):
        i = i+1
    if i == len(A):
        finish = 1
        return finish, A
    else:
        finish = 0
        A[i] = 1
        return finish, A


def NeighborSearch(k, tau, G, D, m, kappa, beta, Max_g):  # D is fixed, only change G
    G_opt = list()
    exp_time1 = Compute_Exp_detect_time(k, tau, G, D, m, kappa, beta)
    exp_time_opt = exp_time1
    G_opt = copy.deepcopy(G)

    last_move_is_ok = 0
    maximum_iteration = tau*k*m
    for i in range(maximum_iteration):
        G2 = copy.deepcopy(G)
        if(i == 0 or (last_move_is_ok == 0 and randint(0, 100) > 50)):
            index1 = randint(0, k-1)
            index2 = index1

            while(index1 == index2):
                index2 = randint(0, k-1)
        if(G2[index1] < Max_g and G2[index2] > 1):
            G2[index1] = G2[index1]+1
            G2[index2] = G2[index2]-1
            exp_time2 = Compute_Exp_detect_time(k, tau, G2, D, m, kappa, beta)
            if(exp_time2 < exp_time1):
                G = copy.deepcopy(G2)
                exp_time1 = exp_time2
                last_move_is_ok = 1

                if(exp_time2 < exp_time_opt):
                    exp_time_opt = exp_time2
                    G_opt = copy.deepcopy(G2)
            else:
                last_move_is_ok = 0
        else:
            last_move_is_ok = 0

    return G_opt


def Heuristic_Search(k, tau, m, mainG, mainD, Max_g, beta, kappa):
    G_opt = list()
    D_opt = list()
    exp_time_opt = tau+1
    Changable = list()
    A = list()
    for i in range(0, k):
        A.append(0)
        if math.floor(mainD[i]) == mainD[i]:
            Changable.append(0)
        else:
            Changable.append(1)

    finish = 0
    while finish == 0:
        G = copy.deepcopy(mainG)
        D = copy.deepcopy(mainD)
        for i in range(0, k):
            if(A[i] == 0):
                D[i] = math.floor(mainD[i])
            else:
                D[i] = math.ceil(mainD[i])

        next = 0
        for i in range(0, k-1):
            if D[i] == D[i+1]:
                next = 1

        if next == 0:
            G = NeighborSearch(k, tau, G, D, m, kappa, beta, Max_g)
            exp_time = Compute_Exp_detect_time(k, tau, G, D, m, kappa, beta)
            if(exp_time < exp_time_opt):
                exp_time_opt = exp_time
                G_opt = copy.deepcopy(G)
                D_opt = copy.deepcopy(D)

        finish, A = Next_List(Changable, A)

    return G_opt, D_opt


def Optimal_Configuration_Heuristic(k, tau, m, Max_g, beta, kappa):
    G = list()                 # size of each group
    D = list()                  # testing day of each group

    if tau % k == 0:
        G, D = Optimal_Configuration(k, tau, m)
        return G, D

    # else
    size = m/k
    s = 0
    for i in range(1, k+1):
        d = (i/k)*tau
        D.append(d)
        G.append(math.floor(size))
        s = s+G[i-1]
        if(s+1 <= i*(m/k)):
            G[i-1] = G[i-1]+1
            s = s+1

    G = Refine(G, m)
    G, D = Heuristic_Search(k, tau, m, G, D, Max_g, beta, kappa)

    return G, D


def retirement_calculation(m, n, p_time, t_time, p, kappa, beta, max_g, max_tau):
    max_interval = max_tau
    min_exp_time = max_interval+1
    k_min = -1
    tau_min = -1
    G_min = list()
    D_min = list()
    #print(m, n, p_time, t_time, p, kappa, beta, max_g, max_tau)
    percent_staff_time = False

    for tau in range(1, max_interval+1):          # test interval
        for k in range(1, tau+1):               # number of groups
            if((k*p_time+m*t_time <= p*n*tau) and (m/k <= max_g)):

                G = list()                 # size of each group
                D = list()                 # testing day of each group
                G, D = Optimal_Configuration_Heuristic(
                    k, tau, m, max_g, beta, kappa)

                exp_time = Compute_Exp_detect_time(
                    k, tau, G, D, m, kappa, beta)

                if exp_time < min_exp_time:
                    min_exp_time = exp_time
                    k_min = k
                    tau_min = tau
                    G_min = G
                    D_min = D
                    percent_staff_time = (k*p_time+m*t_time)/(n*tau)

                    #print(k, t_time, n, tau)
                    # print(min_exp_time, k_min, tau_min,
                    #      G_min, D_min, percent_staff_time)
                    # print('solution found', (k*p_time+m*t_time)/(n*tau))

    if(min_exp_time == max_interval+1):
        min_exp_time = -1

    #staff_time = k*p_time+m*t_time
    #percent_staff_time = (k*p_time+m*t_time)/(n*tau)

    print(k, t_time, n, tau)

    return min_exp_time, k_min, tau_min, G_min, D_min, percent_staff_time


# Input parameters: m, n, p_time, t_time, p, kappa, beta, max_g, max_tau
# exp_time_out, k_out, tau_out, G_out, D_out
#   = retirement_calculation(120, 30, 0.375, 0.03125, 0.1, 10, 0.1, 120, 5)
if __name__ == "__main__":
    import sys

    if len(sys.argv) < 10:
        outText = f'Please insert 9 required parameters on {sys.argv[0]}'
        sys.exit(outText)

    p1 = int(sys.argv[1])
    p2 = int(sys.argv[2])
    p3 = float(sys.argv[3])
    p4 = float(sys.argv[4])
    p5 = float(sys.argv[5])
    p6 = int(sys.argv[6])
    p7 = float(sys.argv[7])
    p8 = int(sys.argv[8])
    p9 = int(sys.argv[9])

    result = retirement_calculation(
        p1, p2, p3, p4, p5, p6, p7, p8, p9)

    exp_time_out, k_out, tau_out, G_out, D_out = result

    print(result)
