from cmath import isnan
import os
import pandas as pd
import numpy as np
import datetime
import math

try:
    DEBUG = os.environ['DEBUG']
except KeyError:
    DEBUG = False

all_gems_df = pd.read_csv(
    'wastewater_source/data/saxony_gemeinde.csv', dtype={"region": object, "wildan-id": object})

all_kreis_df = pd.read_csv(
    'wastewater_source/data/saxony_kreis.csv', dtype={"region": object})

dist_df = pd.read_csv(
    'wastewater_source/data/tdistances.csv')

dist_df.set_index(dist_df.columns[0], inplace=True)
dist_df.index = dist_df.index.map(str)


gem_30_ns = {mid1: [mid2 for mid2 in dist_df.index[:] if dist_df.at[str(mid2),
                                                                    str(mid1)] <= 1800] for mid1 in dist_df.index[:]}
gem_60_ns = {mid1: [mid2 for mid2 in dist_df.index[:] if dist_df.at[mid2,
                                                                    mid1] <= 3600] for mid1 in dist_df.index[:]}
list_gems = all_gems_df.values.tolist()

gem_codes = {
    all_gems_df.loc[gi]['wildan-id']: all_gems_df.loc[gi]['region'] for gi in all_gems_df.index
}

gem_codes_r = {
    all_gems_df.loc[gi]['region']: all_gems_df.loc[gi]['wildan-id'] for gi in all_gems_df.index
}

gem_id_pop = {
    all_gems_df.loc[gi]['region']: all_gems_df.loc[gi]['population'] for gi in all_gems_df.index
}

gem_id_kreis = {
    all_gems_df.loc[gi]['region']: all_gems_df.loc[gi]['kreis'] for gi in all_gems_df.index
}
kreis_id_pop = {
    all_kreis_df.loc[gi]['region']: all_kreis_df.loc[gi]['population'] for gi in all_kreis_df.index
}


# print(list_gems[0])


def wastewater_heatmap(df_ww, df_cases, radius, metric):
    """_summary_

    Args:
        df_ww (_type_): _description_
        df_cases (_type_): _description_
        radius (_type_): _description_
        metric (_type_): _description_

    Returns:
        _type_: _description_
    """

    data_hm = {}

    print('df_ww', df_ww)
    ids_ww = df_ww['plant'].unique()

    data_cases_all = df_cases.to_dict('records')

    for id_ww in ids_ww:
        r_vals = {}
        ww_rid = str(gem_id_kreis[id_ww]) if radius == 'kreis' else id_ww

        # kreis
        if radius == 'kreis':
            kreis_pop = kreis_id_pop[ww_rid]

            for r_record in data_cases_all:
                # print(ww_kreis, r_record['region'])
                if str(r_record['region']) == str(ww_rid):
                    if metric == 'inc':
                        r_vals[str(r_record['date'])] = int((
                            r_record['cases'] / kreis_pop) * 100000)

        # gemeinde
        else:
            r_ids = gem_30_ns[id_ww] if radius == "30" else gem_60_ns[id_ww]
            if metric == 'inc':
                cases_sums = {}
                pops_sums = {}
                for r_record in data_cases_all:
                    if r_record['region'] in r_ids:
                        r_date = str(r_record['date'])
                        r_pop = gem_id_pop[r_record['region']]
                        r_cases = r_record['cases']
                        if r_date in cases_sums:
                            cases_sums[r_date] += r_cases
                            pops_sums[r_date] += r_pop
                        else:
                            cases_sums[r_date] = r_cases
                            pops_sums[r_date] = r_pop

                for date in cases_sums.keys():
                    r_vals[date] = int(
                        (cases_sums[date] / pops_sums[date]) * 100000)

        ww_vals = {}
        df_ww_sub = df_ww[df_ww['plant'] == id_ww]
        data_ww = df_ww_sub.to_dict('records')
        for ww_record in data_ww:
            if math.isnan(float(ww_record['c_e'])):
                # print('this is nan', ww_record['date'])
                ww_vals[str(ww_record['date'])] = False
            else:
                ww_vals[str(ww_record['date'])] = ww_record['c_e']

        data_hm[ww_rid] = {
            'ww_vals': ww_vals,
            'r_vals': r_vals,
        }

    return data_hm


def wastewater_timeline(df, selection):
    """_summary_

    Args:
        df (_type_): _description_
        radius (_type_): _description_
        metric (_type_): _description_

    Returns:
        _type_: _description_
    """
    df = df.set_index('region')

    vls = {gem_codes[str(rid)] if str(rid) in gem_codes else '0': int(
        df.at[rid, 'cases']) if list(df.index).count(rid) == 1 else 0 for rid in df.index}
    return vls


def wastewater_map(df_ww, df_cases, radius, metric, date):
    """_summary_

    Args:
        df_plants (_type_): _description_
        df_gems (_type_): _description_
        radius (_type_): _description_
        mode (_type_): _description_
        metric (_type_): _description_

    Returns:
        _type_: _description_
    """

    print(df_ww)
    print(df_cases)
    data_map = {
        "region": {},
        "ww": {ww['plant']: False if math.isnan(ww['c_e']) else ww['c_e'] for ww in df_ww.to_dict('records')}
    }

    data_cases = df_cases.to_dict('records')
    if metric == 'inc':
        for r_cases in data_cases:
            pop = kreis_id_pop[r_cases['region']
                               ] if radius == 'kreis' else gem_id_pop[r_cases['region']]
            if pop and r_cases['cases']:
                inc = int((r_cases['cases'] / pop) * 100000)
                data_map['region'][r_cases['region']
                                   ] = False if math.isnan(inc) else inc
            else:
                data_map['region'][r_cases['region']] = False

    return data_map
